@@ -1,5 +1,5 @@
 # codegen.py
-# Copyright (C) 2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -13,6 +13,7 @@ from mako import util, ast, parsetree, filters
 
 MAGIC_NUMBER = 1
 
+
 def compile(node, uri, filename=None):
     """generate module source code given a parsetree node, uri, and optional source filename"""
     buf = util.FastEncodingBuffer()
@@ -117,7 +118,7 @@ class _GenerateRenderMethod(object):
         module_identifiers.topleveldefs = module_identifiers.topleveldefs.union(main_identifiers.topleveldefs)
         [module_identifiers.declared.add(x) for x in ["UNDEFINED"]]
         self.compiler.identifiers = module_identifiers
-        self.printer.writeline("_exports = %s" % repr([n.name for n in main_identifiers.topleveldefs]))
+        self.printer.writeline("_exports = %s" % repr([n.name for n in main_identifiers.topleveldefs.values()]))
         self.printer.write("\n\n")
 
         if len(module_code):
@@ -129,7 +130,7 @@ class _GenerateRenderMethod(object):
         elif len(namespaces):
             self.write_namespaces(namespaces)
 
-        return main_identifiers.topleveldefs
+        return main_identifiers.topleveldefs.values()
 
     def write_render_callable(self, node, name, args, buffered, filtered, cached):
         """write a top-level render callable.
@@ -204,7 +205,7 @@ class _GenerateRenderMethod(object):
                 callable_name = "make_namespace()"
             else:
                 callable_name = "None"
-            self.printer.writeline("ns = runtime.Namespace(%s, context._clean_inheritance_tokens(), templateuri=%s, callables=%s, calling_uri=_template_uri)" % (repr(node.name), node.parsed_attributes.get('file', 'None'), callable_name))
+            self.printer.writeline("ns = runtime.Namespace(%s, context._clean_inheritance_tokens(), templateuri=%s, callables=%s, calling_uri=_template_uri, module=%s)" % (repr(node.name), node.parsed_attributes.get('file', 'None'), callable_name, node.parsed_attributes.get('module', 'None')))
             if eval(node.attributes.get('inheritable', "False")):
                 self.printer.writeline("context['self'].%s = ns" % (node.name))
             self.printer.writeline("context.namespaces[(__name__, %s)] = ns" % repr(node.name))
@@ -228,14 +229,13 @@ class _GenerateRenderMethod(object):
         
         # collection of all defs available to us in this scope
         comp_idents = dict([(c.name, c) for c in identifiers.defs])
-
         to_write = util.Set()
         
         # write "context.get()" for all variables we are going to need that arent in the namespace yet
         to_write = to_write.union(identifiers.undeclared)
         
         # write closure functions for closures that we define right here
-        to_write = to_write.union(util.Set([c.name for c in identifiers.closuredefs]))
+        to_write = to_write.union(util.Set([c.name for c in identifiers.closuredefs.values()]))
 
         # remove identifiers that are declared in the argument signature of the callable
         to_write = to_write.difference(identifiers.argument_declared)
@@ -344,15 +344,22 @@ class _GenerateRenderMethod(object):
         self.printer.writeline("__%s = %s" % (name, name))
         cachekey = node_or_pagetag.parsed_attributes.get('cache_key', repr(name))
         cacheargs = {}
-        for arg in (('cache_type', 'type'), ('cache_dir', 'data_dir')):
+        print node_or_pagetag
+        for arg in (('cache_type', 'type'), ('cache_dir', 'data_dir'), ('cache_timeout', 'expiretime')):
             val = node_or_pagetag.parsed_attributes.get(arg[0], None)
             if val is not None:
-                cacheargs[arg[1]] = val
+                if arg[1] == 'expiretime':
+                    cacheargs[arg[1]] = int(eval(val))
+                else:
+                    cacheargs[arg[1]] = val
             else:
                 if self.compiler.pagetag is not None:
                     val = self.compiler.pagetag.parsed_attributes.get(arg[0], None)
                     if val is not None:
-                        cacheargs[arg[1]] = val
+                        if arg[1] == 'expiretime':
+                            cacheargs[arg[1]] == int(eval(val))
+                        else:
+                            cacheargs[arg[1]] = val
             
         self.printer.writeline("def %s(context, *args, **kwargs):" % name)
 
@@ -377,15 +384,22 @@ class _GenerateRenderMethod(object):
         if self.compiler.pagetag:
             args += self.compiler.pagetag.filter_args.args
         for e in args:
-            e = d.get(e, e)
+            # if filter given as a function, get just the identifier portion
+            m = re.match(r'(.+?)(\(.*\))', e)
+            if m:
+                (ident, fargs) = m.group(1,2)
+                f = d.get(ident, ident)
+                e = f + fargs
+            else:
+                e = d.get(e, e)
             target = "%s(%s)" % (e, target)
         return target
         
     def visitExpression(self, node):
         self.write_source_comment(node)
         if len(node.escapes) or (self.compiler.pagetag is not None and len(self.compiler.pagetag.filter_args.args)):
-            s = self.create_filter_callable(node.escapes_code.args, node.text)
-            self.printer.writeline("context.write(unicode(%s))" % s)
+            s = self.create_filter_callable(node.escapes_code.args, "unicode(%s)" % node.text)
+            self.printer.writeline("context.write(%s)" % s)
         else:
             self.printer.writeline("context.write(unicode(%s))" % node.text)
             
@@ -423,7 +437,7 @@ class _GenerateRenderMethod(object):
             if not self.in_def and len(self.identifiers.locally_assigned) > 0:
                 # if we are the "template" def, fudge locally declared/modified variables into the "__locals" dictionary,
                 # which is used for def calls within the same template, to simulate "enclosing scope"
-                self.printer.writeline('__locals.update(dict([(k, v) for k, v in locals().iteritems() if k in [%s]]))' % ','.join([repr(x) for x in node.declared_identifiers()]))
+                self.printer.writeline('__locals.update(dict([(k, locals()[k]) for k in [%s]]))' % ','.join([repr(x) for x in node.declared_identifiers()]))
                 
     def visitIncludeTag(self, node):
         self.write_source_comment(node)
@@ -448,6 +462,9 @@ class _GenerateRenderMethod(object):
             def visitDefTag(s, node):
                 self.write_inline_def(node, callable_identifiers, nested=False)
                 export.append(node.name)
+                # remove defs that are within the <%call> from the "closuredefs" defined
+                # in the body, so they dont render twice
+                del body_identifiers.closuredefs[node.name]
         vis = DefVisitor()
         for n in node.nodes:
             n.accept_visitor(vis)
@@ -493,7 +510,7 @@ class _Identifiers(object):
     def __init__(self, node=None, parent=None, nested=False):
         if parent is not None:
             # things that have already been declared in an enclosing namespace (i.e. names we can just use)
-            self.declared = util.Set(parent.declared).union([c.name for c in parent.closuredefs]).union(parent.locally_declared).union(parent.argument_declared)
+            self.declared = util.Set(parent.declared).union([c.name for c in parent.closuredefs.values()]).union(parent.locally_declared).union(parent.argument_declared)
             
             # if these identifiers correspond to a "nested" scope, it means whatever the 
             # parent identifiers had as undeclared will have been declared by that parent, 
@@ -502,10 +519,10 @@ class _Identifiers(object):
                 self.declared = self.declared.union(parent.undeclared)
             
             # top level defs that are available
-            self.topleveldefs = util.Set(parent.topleveldefs)
+            self.topleveldefs = util.SetLikeDict(**parent.topleveldefs)
         else:
             self.declared = util.Set()
-            self.topleveldefs = util.Set()
+            self.topleveldefs = util.SetLikeDict()
         
         # things within this level that are referenced before they are declared (e.g. assigned to)
         self.undeclared = util.Set()
@@ -522,7 +539,7 @@ class _Identifiers(object):
         self.argument_declared = util.Set()
         
         # closure defs that are defined in this level
-        self.closuredefs = util.Set()
+        self.closuredefs = util.SetLikeDict()
         
         self.node = node
         
@@ -532,11 +549,11 @@ class _Identifiers(object):
     def branch(self, node, **kwargs):
         """create a new Identifiers for a new Node, with this Identifiers as the parent."""
         return _Identifiers(node, self, **kwargs)
-        
-    defs = property(lambda s:s.topleveldefs.union(s.closuredefs))
+    
+    defs = property(lambda self:util.Set(self.topleveldefs.union(self.closuredefs).values()))
     
     def __repr__(self):
-        return "Identifiers(declared=%s, locally_declared=%s, undeclared=%s, topleveldefs=%s, closuredefs=%s, argumenetdeclared=%s)" % (repr(list(self.declared)), repr(list(self.locally_declared)), repr(list(self.undeclared)), repr([c.name for c in self.topleveldefs]), repr([c.name for c in self.closuredefs]), repr(self.argument_declared))
+        return "Identifiers(declared=%s, locally_declared=%s, undeclared=%s, topleveldefs=%s, closuredefs=%s, argumenetdeclared=%s)" % (repr(list(self.declared)), repr(list(self.locally_declared)), repr(list(self.undeclared)), repr([c.name for c in self.topleveldefs.values()]), repr([c.name for c in self.closuredefs.values()]), repr(self.argument_declared))
         
     def check_declared(self, node):
         """update the state of this Identifiers with the undeclared and declared identifiers of the given node."""
@@ -561,9 +578,9 @@ class _Identifiers(object):
             self.locally_assigned = self.locally_assigned.union(node.declared_identifiers())
     def visitDefTag(self, node):
         if node.is_root():
-            self.topleveldefs.add(node)
+            self.topleveldefs[node.name] = node
         elif node is not self.node:
-            self.closuredefs.add(node)
+            self.closuredefs[node.name] = node
         for ident in node.undeclared_identifiers():
             if ident != 'context' and ident not in self.declared.union(self.locally_declared):
                 self.undeclared.add(ident)
